#ifndef ANCHORS_HPP
#define ANCHORS_HPP

#include <vector>
#include <iostream>
#include <queue>
#include <algorithm>
#include "graph.hpp"
#include "gtruth.hpp"

template<typename Vertex, typename Edge, typename EType>
class Anchors{
private:
	std::vector< Vertex > anchors;
	struct Order
	{
	    bool operator()(EType const& a, EType const& b) const
	    {
	        return a.second < b.second;
	    }
	};

public:
	Anchors( ) { }

	Anchors( int k, Graph<Vertex, Edge, EType>& g, GTruth<Vertex>& gt ) {
		degBasedAnchors( k, g, gt );
	}

	Anchors( std::vector< Vertex >& anchorsA, GTruth<Vertex>& gt ) {
		for( int i=0 ; i<anchorsA.size() ; i++ ) {
			auto a = anchorsA[i];
			anchors.push_back( gt.getMapOf(a) );
		}
	}

	void degBasedAnchors( int k, Graph<Vertex, Edge, EType>&g, GTruth<Vertex>& gt ) {
		std::priority_queue<EType, std::vector<EType>, Order > q;
		int n = g.nVert();

		for( int i=0 ; i<n ; i++ ) {
			if( gt.getMapOf( i ) < 0 ) continue;

			int deg = g.getDeg(i);
			if( q.size() < k ) {
				q.push( EType(i, deg) );
			}
			else if( q.top().second < deg ) {
				q.pop();
				q.push( EType(i, deg) );
			} else {
				continue;
			}
		}

		while( !q.empty() && k>0) {
			anchors.push_back( q.top().first );
			q.pop();
			--k;
		}
	}

	std::vector<Vertex>& getAnchors() {
		return anchors;
	};

	Vertex getAnchor( int i ) {
		return anchors[i];
	};

	void addAnchor( Vertex v ) {
		anchors.push_back( v );
	}
	int getAnchorSize() {
		return anchors.size();
	}

	void eraseLastAnchor() {
		anchors.pop_back();
	}
};


#endif