#ifndef GRAPH_H
#define GRAPH_H

#include <algorithm>
#include <vector>
#include <list>

template<typename Vertex, typename Edge, typename EType>
class Graph{
private:
	unsigned int E, V;
	int nType, nEType;
	std::vector< std::vector<Edge> > adjList;
	std::vector< std::string > attributes;
	std::vector< int > vTypes;
	std::vector< std::vector<EType> > eTypes;
	std::vector< std::vector<int> > distances;
	std::vector< std::vector<int> > sTypes;
	std::vector< std::vector<int> > svTypes;
	std::vector< std::vector<float> > sWeights;

public:
	Graph() {
		E=0; V=0; nType=0, nEType=0;
	}

	unsigned int nEdge() const {
		return E;
	}

	unsigned int nVert() const {

		return static_cast<unsigned int>(adjList.size());
	}

	std::vector<Edge>& adj(Vertex v) {
		return adjList[v];
	}

	void addEdge(Vertex v, Edge e) {
		Vertex w = e.first;
		if( std::max(v,w) >= adjList.size() ) {
			adjList.resize( std::max(v,w) + 1 );
		}

		adjList[v].push_back( e );
		++E;
	}

	float getEdgeWeight( Vertex v, Edge e ) {
		return 1;//Weights[v];
	}

	std::vector<float>& getSEWeightList( Vertex v ) {
		return sWeights[v];
	}

	int getEdgeType(Vertex v, Edge e) {
		return 1;//(int)Weights[std::min(u,v)][std::max(u,v)];
	}

	int getVType( Vertex v ) {
		return vTypes[v];
	}

	std::vector<int>& getSETypeList( Vertex v ) {
		return sTypes[v];
	}

	std::vector<int>& getSVTypeList( Vertex v ) {
		return svTypes[v];
	}

	void setAttribute(Vertex v, std::string attr) {
		if( attributes.size() < adjList.size() ){
			attributes.resize( adjList.size() );
		}

		attributes[v] = attr;
	}

	bool isAttributed() {
		return ( attributes.size() > 0 );
	}

	void setVType(Vertex v, int t) {
		if( vTypes.size() < adjList.size() ){
			vTypes.resize( adjList.size() );
		}

		nType = std::max( nType, t );
		vTypes[v] = t;
	}

	void setEType(Vertex v, EType t) {
		if( eTypes.size() < adjList.size() ){
			eTypes.resize( adjList.size() );
		}

		nEType = std::max( nEType, t.second );
		eTypes[v].push_back( t );
	}

	std::string& getAttribute( Vertex v ) {
		return attributes[v];
	}

	std::string* getAttribute2( Vertex v ) {
		return &attributes[v];
	}

	int getDeg(Vertex v) {
		return adjList[v].size();
	}

	int getNType() {
		return nType+1;
	}

	int getNEType() {
		return nEType+1;
	}

	void BFS( Vertex s, int o ) {
		int n = nVert();
		bool *visited = new bool[ n ];
		for(int i = 0; i < n; i++) {
			visited[i] = false;
		}
		std::vector< int > temp(n,2<<22);

		std::list<Vertex> queue;

		setDistance( o, s, 0 );
		visited[s] = true;
		queue.push_back( s );

		while( !queue.empty() ) {
			s = queue.front();
			queue.pop_front();

			for( int j=0 ; j<adj(s).size() ; j++ ) {
				auto i = adj(s)[j];
				if( !visited[i.first] ) {
					visited[i.first] = true;
					queue.push_back( i.first );
					setDistance( o, i.first, getDistance(o,s)+1 );
				}
			}
		}
		free(visited);
	}

	void setSTypes() {
		sTypes.resize( adjList.size(), std::vector<int>(0) );
		for( int i=0 ; i<adjList.size() ; i++ ) {
			for( int j=0 ; j<eTypes[i].size() ; j++ ) {
				auto t = eTypes[i][j];
				sTypes[i].push_back( t.second );
			}
			std::sort( sTypes[i].begin(), sTypes[i].end() );
		}
	}

	void setSWeights() {
		sWeights.resize( adjList.size(), std::vector<float>(0) );
		svTypes.resize( adjList.size(), std::vector<int>(0) );
		for( int i=0 ; i<adjList.size() ; i++ ) {
			for( int j=0 ; j<adjList[i].size() ; j++ ) {
				auto t = adjList[i][j];
				sWeights[i].push_back( t.second );
				svTypes[i].push_back( vTypes[t.first] );
			}
			std::sort( sWeights[i].begin(), sWeights[i].end() );
			std::sort( svTypes[i].begin(), svTypes[i].end() );
		}
	}

	void setDistance( int o, Vertex v, int d ) {
		distances[o][v] = d;
	}

	int getDistance( int o, Vertex v ) {
		return distances[o][v];
	}

	std::vector<int>& getDistanceVector( int o ) {
		return distances[o];
	}
	void prepDistanceMatrix( int k ) {
		distances.resize( k, std::vector<int>(adjList.size()) );
#ifdef M_OMP
		#pragma omp parallel for schedule(dynamic)
		for( int i=0 ; i<k ; i++ ) {
#elif M_CILK
		cilk_for( int i=0 ; i<k ; i++ ) {
#else
		for( int i=0 ; i<k ; i++ ) {
#endif
			for( int j=0 ; j<adjList.size() ; j++ ) {
				distances[i][j] = 2<<22;
			}
		}
	}

	void setDistanceMatrix( std::vector<Vertex>& a ) {
		int k = a.size();
		prepDistanceMatrix( k );
#ifdef M_OMP
		#pragma omp parallel for schedule(dynamic)
		for( int i=0 ; i<a.size() ; i++ ) {
#elif M_CILK
		cilk_for( int i=0 ; i<a.size() ; i++ ) {
#else
		for( int i=0 ; i<a.size() ; i++ ) {
#endif
			BFS( a[i], i );
		}
	}

	void addDistanceVector( Vertex v ) {
		int o = distances.size();
		distances.push_back( std::vector<int>(adjList.size(), 2<<22) );
	}

	void eraseLastAnchor() {
		distances.pop_back();
	}
};

#endif