#ifndef GTRUTH_H
#define GTRUTH_H

#include <vector>
#include <iostream>
#include <unordered_map>


template<typename Vertex>
class GTruth{
private:
	using VVector = std::vector< Vertex >;
	using BNeighbors = std::vector< VVector* >;
	using Task = std::pair< VVector*, BNeighbors >;
	using LTask = std::vector< Task >;
	using sMap = std::vector< std::pair<Vertex, float> >;

	std::unordered_map<Vertex, bool> hitMap;
	std::vector< Vertex > mu;
	std::string fname;
public:
	GTruth( ){}
	GTruth( std::string fname, int n ) {
		setGTruth( fname, n );
	}

	void setGTruth( std::string fname, int n ) {
		std::fstream gIn(fname);
		Vertex u=0 , v;
		mu.resize( n );

		while( gIn >> v ) {
			mu[u] = v;
			++u;
		}
	}

	/* Get map of u \in V1 in V2 */
	Vertex getMapOf(Vertex v) {
		return mu[v];
	}

	Vertex getMappedFrom(Vertex v) {
		Vertex result = -1;
		int i=0;

		while( i<mu.size() && result==-1 ) {
			if( mu[i] == v ) {
				result = i;
			}
			++i;
		}

		return result;
	}

	int getRecall( sMap& m ) {
		int match = 0;
		for( int i=0 ; i<m.size() ; i++ ) {
			match += ( m[i].first == mu[i] && mu[i]>=0 );
		}

		return match;
	}
};


#endif